from PyQt6.QtWidgets import QWidget, QVBoxLayout, QApplication
from PyQt6.QtGui import QColor, QFont
from PyQt6.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs
import sys
import os


class CodeEditor(QWidget):
    def __init__(self, parent=None, project_path=None):
        super().__init__(parent)
        self.project_path = project_path

        # Create layout
        layout = QVBoxLayout()
        self.setLayout(layout)

        # Create QScintilla editor
        self.editor = QsciScintilla()
        layout.addWidget(self.editor)

        # Set the default font
        font = QFont("Consolas", 11)
        self.editor.setFont(font)

        # Set the default theme (dark theme)
        self.set_dark_theme()

        # Enable line numbers
        self.editor.setMarginType(0, QsciScintilla.MarginType.NumberMargin)
        self.editor.setMarginWidth(0, "000")

        # Enable folding
        self.editor.setFolding(QsciScintilla.FoldStyle.BoxedTreeFoldStyle)

        # Set Python lexer
        self.lexer = QsciLexerPython()
        self.lexer.setFont(font)
        self.editor.setLexer(self.lexer)

        # Enable auto-indentation
        self.editor.setAutoIndent(True)
        self.editor.setIndentationGuides(True)
        self.editor.setTabWidth(4)
        self.editor.setIndentationsUseTabs(False)

        # Enable brace matching
        self.editor.setBraceMatching(QsciScintilla.BraceMatch.StrictBraceMatch)

        # Set up code completion
        self.setup_code_completion()

        # Enable auto-completion
        self.editor.setAutoCompletionSource(
            QsciScintilla.AutoCompletionSource.AcsAll)
        self.editor.setAutoCompletionThreshold(2)
        self.editor.setAutoCompletionCaseSensitivity(False)
        self.editor.setAutoCompletionReplaceWord(True)

    def set_dark_theme(self):
        """Set dark theme colors"""
        # Editor background
        self.editor.setColor(QColor("#FFFFFF"))
        self.editor.setPaper(QColor("#1E1E1E"))
        self.editor.setMarginsBackgroundColor(QColor("#1E1E1E"))
        self.editor.setMarginsForegroundColor(QColor("#CCCCCC"))

        # Selection colors
        self.editor.setSelectionBackgroundColor(QColor("#264F78"))
        self.editor.setSelectionForegroundColor(QColor("#FFFFFF"))

        # Caret color
        self.editor.setCaretForegroundColor(QColor("#FFFFFF"))
        self.editor.setCaretLineVisible(True)
        self.editor.setCaretLineBackgroundColor(QColor("#2C2C2C"))

    def setup_code_completion(self):
        """Set up code completion with project-aware suggestions"""
        self.api = QsciAPIs(self.lexer)

        # Add Python builtins
        python_keywords = ["def", "class", "for", "while", "if", "else", "try", "except",
                           "import", "from", "as", "return", "yield", "break", "continue",
                           "pass", "raise", "with", "in", "is", "not", "and", "or"]

        for keyword in python_keywords:
            self.api.add(keyword)

        # If project path is provided, scan for additional completions
        if self.project_path:
            self.scan_project_for_completions()

        self.api.prepare()

    def scan_project_for_completions(self):
        """Scan project directory for Python files and extract symbols for completion"""
        if not self.project_path or not os.path.exists(self.project_path):
            return

        for root, _, files in os.walk(self.project_path):
            for file in files:
                if file.endswith('.py'):
                    try:
                        with open(os.path.join(root, file), 'r', encoding='utf-8') as f:
                            content = f.read()
                            # Basic symbol extraction (this can be enhanced)
                            for line in content.split('\n'):
                                line = line.strip()
                                if line.startswith('def '):
                                    func_name = line[4:].split('(')[0]
                                    self.api.add(func_name)
                                elif line.startswith('class '):
                                    class_name = line[6:].split(
                                        '(')[0].split(':')[0]
                                    self.api.add(class_name)
                    except Exception:
                        continue

    def set_text(self, text):
        """Set the editor text"""
        self.editor.setText(text)

    def get_text(self):
        """Get the editor text"""
        return self.editor.text()

    def set_file(self, file_path):
        """Load and set text from a file"""
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                self.set_text(f.read())
        except Exception as e:
            print(f"Error loading file: {e}")

    def save_file(self, file_path):
        """Save current text to a file"""
        try:
            with open(file_path, 'w', encoding='utf-8') as f:
                f.write(self.get_text())
            return True
        except Exception as e:
            print(f"Error saving file: {e}")
            return False
